<?php

namespace Kylin\Sms\Gateways;

use Kylin\Sms\Exceptions\GatewayException;

/**
 * SendCloud 短信网关
 * 
 * 支持 SendCloud 短信服务的发送功能
 * SendCloud 是国内领先的云端邮件与短信发送平台
 * 
 * 文档：http://www.sendcloud.net/doc/sms/
 * 
 * @package Kylin\Sms\Gateways
 */
class SendcloudGateway extends BaseGateway
{
    /**
     * 网关名称标识
     */
    protected static string $name = 'sendcloud';

    /**
     * API 请求地址
     */
    protected const API_URL = 'https://api.sendcloud.net/apiv2/smssend';

    /**
     * SendCloud SMS 用户名
     * 
     * @var string
     */
    protected string $smsUser;

    /**
     * SendCloud SMS 密钥
     * 
     * @var string
     */
    protected string $smsKey;

    /**
     * 短信签名
     * 
     * @var string
     */
    protected string $signName;

    /**
     * 构造函数
     * 
     * 初始化 SendCloud 短信网关配置
     * 
     * @param array $config 网关配置数组
     */
    public function __construct(array $config = [])
    {
        parent::__construct($config);
        $this->smsUser = $config['sms_user'] ?? '';
        $this->smsKey = $config['sms_key'] ?? '';
        $this->signName = $config['sign_name'] ?? '';
    }

    /**
     * 发送短信
     * 
     * 发送短信到指定手机号
     * 
     * @param string $to 接收手机号
     * @param string $content 短信内容（SendCloud 使用模板，此参数可忽略）
     * @param array $data 附加数据，包含 template_id 和 template_params
     * @return array 发送结果
     */
    public function send(string $to, string $content, array $data = []): array
    {
        $params = [
            'smsUser' => $this->smsUser,
            'templateId' => $data['template_id'] ?? '',
            'phone' => $this->formatPhone($to),
            'vars' => json_encode($data['template_params'] ?? []),
        ];

        $params['signature'] = $this->computeSignature($params);

        try {
            $response = $this->request('POST', self::API_URL, [
                'form_params' => $params,
            ]);
            return $this->parseResponse($response);
        } catch (GatewayException $e) {
            return $this->buildErrorResponse($e->getMessage(), $e->getCode(), $e->getContext());
        }
    }

    /**
     * 发送模板短信
     * 
     * 使用预定义模板发送短信
     * 
     * @param string $to 接收手机号
     * @param string $templateId 模板 ID
     * @param array $params 模板参数
     * @return array 发送结果
     */
    public function sendTemplate(string $to, string $templateId, array $params = []): array
    {
        return $this->send($to, '', [
            'template_id' => $templateId,
            'template_params' => $params,
        ]);
    }

    /**
     * 批量发送短信
     * 
     * 向多个手机号发送相同内容的短信
     * 
     * @param array $recipients 接收手机号数组
     * @param string $content 短信内容
     * @param array $data 附加数据
     * @return array 发送结果
     */
    public function sendBatch(array $recipients, string $content, array $data = []): array
    {
        $results = [];
        foreach ($recipients as $recipient) {
            $results[] = $this->send($recipient, $content, $data);
        }
        return $this->buildSuccessResponse(['results' => $results]);
    }

    /**
     * 计算签名
     * 
     * 生成 SendCloud API 所需的签名
     * 
     * @param array $params 请求参数
     * @return string 签名字符串
     */
    protected function computeSignature(array $params): string
    {
        ksort($params);
        $queryString = '';
        foreach ($params as $key => $value) {
            $queryString .= $key . '=' . $value . '&';
        }
        $queryString = rtrim($queryString, '&');
        return md5($this->smsKey . '&' . $queryString . '&' . $this->smsKey);
    }

    /**
     * 解析响应结果
     * 
     * 将 SendCloud API 响应转换为标准化的结果格式
     * 
     * @param array $response API 响应
     * @return array 标准化的响应结果
     */
    protected function parseResponse(array $response): array
    {
        if (isset($response['statusCode']) && $response['statusCode'] === 200) {
            return $this->buildSuccessResponse([
                'info' => $response['info'] ?? [],
            ]);
        }

        return $this->buildErrorResponse(
            $response['message'] ?? '发送失败',
            $response['statusCode'] ?? 500,
            $response
        );
    }
}
