<?php

namespace Kylin\Sms\Gateways;

use Kylin\Sms\Exceptions\GatewayException;

/**
 * 容联云通讯短信网关
 * 
 * 支持容联云通讯短信服务的发送功能
 * 容联云通讯是国内领先的云通讯平台，提供语音、短信、视频等通讯服务
 * 
 * 文档：https://www.yuntongxun.com/doc/ready/demo/1_4_1_2.html
 * 
 * @package Kylin\Sms\Gateways
 */
class RonglianyunGateway extends BaseGateway
{
    /**
     * 网关名称标识
     */
    protected static string $name = 'ronglianyun';

    /**
     * API 请求地址
     */
    protected const API_URL = 'https://app.cloopen.com:8883/2013-12-26/Accounts';

    /**
     * 容联云账户 SID
     * 
     * @var string
     */
    protected string $accountSid;

    /**
     * 容联云认证令牌
     * 
     * @var string
     */
    protected string $authToken;

    /**
     * 应用 ID
     * 
     * @var string
     */
    protected string $appId;

    /**
     * 短信签名
     * 
     * @var string
     */
    protected string $signName;

    /**
     * 构造函数
     * 
     * 初始化容联云通讯短信网关配置
     * 
     * @param array $config 网关配置数组
     */
    public function __construct(array $config = [])
    {
        parent::__construct($config);
        $this->accountSid = $config['account_sid'] ?? '';
        $this->authToken = $config['auth_token'] ?? '';
        $this->appId = $config['app_id'] ?? '';
        $this->signName = $config['sign_name'] ?? '';
    }

    /**
     * 发送短信
     * 
     * 使用模板方式发送短信到指定手机号
     * 
     * @param string $to 接收手机号
     * @param string $content 短信内容（容联云使用模板，此参数可忽略）
     * @param array $data 附加数据，包含 template_id 和 template_params
     * @return array 发送结果
     */
    public function send(string $to, string $content, array $data = []): array
    {
        $timestamp = date('YmdHis');
        $sig = strtoupper(md5($this->accountSid . $this->authToken . $timestamp));
        $url = self::API_URL . '/' . $this->accountSid . '/SMS/TemplateSMS?sig=' . $sig;

        $params = [
            'to' => $this->formatPhone($to),
            'appId' => $this->appId,
            'templateId' => $data['template_id'] ?? '',
            'datas' => array_values($data['template_params'] ?? []),
        ];

        $headers = [
            'Authorization' => base64_encode($this->accountSid . ':' . $timestamp),
            'Content-Type' => 'application/json;charset=utf-8',
            'Accept' => 'application/json',
        ];

        try {
            $response = $this->request('POST', $url, [
                'headers' => $headers,
                'json' => $params,
            ]);
            return $this->parseResponse($response);
        } catch (GatewayException $e) {
            return $this->buildErrorResponse($e->getMessage(), $e->getCode(), $e->getContext());
        }
    }

    /**
     * 发送模板短信
     * 
     * 使用预定义模板发送短信
     * 
     * @param string $to 接收手机号
     * @param string $templateId 模板 ID
     * @param array $params 模板参数
     * @return array 发送结果
     */
    public function sendTemplate(string $to, string $templateId, array $params = []): array
    {
        return $this->send($to, '', [
            'template_id' => $templateId,
            'template_params' => $params,
        ]);
    }

    /**
     * 批量发送短信
     * 
     * 向多个手机号发送相同内容的短信
     * 
     * @param array $recipients 接收手机号数组
     * @param string $content 短信内容
     * @param array $data 附加数据
     * @return array 发送结果
     */
    public function sendBatch(array $recipients, string $content, array $data = []): array
    {
        $results = [];
        foreach ($recipients as $recipient) {
            $results[] = $this->send($recipient, $content, $data);
        }
        return $this->buildSuccessResponse(['results' => $results]);
    }

    /**
     * 解析响应结果
     * 
     * 将容联云 API 响应转换为标准化的结果格式
     * 
     * @param array $response API 响应
     * @return array 标准化的响应结果
     */
    protected function parseResponse(array $response): array
    {
        if (isset($response['statusCode']) && $response['statusCode'] === '000000') {
            return $this->buildSuccessResponse([
                'sms_message_sid' => $response['SMSMessageSid'] ?? '',
                'date_created' => $response['dateCreated'] ?? '',
            ]);
        }

        return $this->buildErrorResponse(
            $response['statusMsg'] ?? '发送失败',
            $response['statusCode'] ?? 500,
            $response
        );
    }
}
