<?php

namespace Kylin\Sms\Exceptions;

/**
 * 网关未找到异常
 * 
 * 当尝试使用未配置或不存在的短信网关时抛出此异常
 * 通常发生在以下情况：
 * - 网关未在配置文件中定义
 * - 网关驱动类不存在
 * - 网关名称拼写错误
 * 
 * @package Kylin\Sms\Exceptions
 */
class GatewayNotFoundException extends SmsException
{
    /**
     * 构造函数
     * 
     * 创建网关未找到异常实例
     * 
     * @param string $message 异常消息，默认为 '短信网关未找到'
     * @param int $code 错误代码，默认为 404
     */
    public function __construct(string $message = '短信网关未找到', int $code = 404)
    {
        parent::__construct($message, $code);
    }
}
