<?php

namespace Kylin\Sms\Gateways;

use Kylin\Sms\Exceptions\GatewayException;

/**
 * UCloud 短信网关
 * 
 * 支持 UCloud 短信服务的发送功能
 * UCloud 是国内领先的云计算服务商，提供稳定可靠的短信发送能力
 * 
 * 文档：https://docs.ucloud.cn/ussms/README
 * 
 * @package Kylin\Sms\Gateways
 */
class UcloudGateway extends BaseGateway
{
    /**
     * 网关名称标识
     */
    protected static string $name = 'ucloud';

    /**
     * API 请求地址
     */
    protected const API_URL = 'https://api.ucloud.cn/';

    /**
     * UCloud 公钥
     * 
     * @var string
     */
    protected string $publicKey;

    /**
     * UCloud 私钥
     * 
     * @var string
     */
    protected string $privateKey;

    /**
     * 项目 ID
     * 
     * @var string
     */
    protected string $projectId;

    /**
     * 短信签名
     * 
     * @var string
     */
    protected string $signName;

    /**
     * 构造函数
     * 
     * 初始化 UCloud 短信网关配置
     * 
     * @param array $config 网关配置数组
     */
    public function __construct(array $config = [])
    {
        parent::__construct($config);
        $this->publicKey = $config['public_key'] ?? '';
        $this->privateKey = $config['private_key'] ?? '';
        $this->projectId = $config['project_id'] ?? '';
        $this->signName = $config['sign_name'] ?? '';
    }

    /**
     * 发送短信
     * 
     * 发送短信到指定手机号
     * 
     * @param string $to 接收手机号
     * @param string $content 短信内容（UCloud 使用模板，此参数可忽略）
     * @param array $data 附加数据，包含 template_id 和 template_params
     * @return array 发送结果
     */
    public function send(string $to, string $content, array $data = []): array
    {
        $params = [
            'Action' => 'SendUSMSMessage',
            'PublicKey' => $this->publicKey,
            'ProjectId' => $this->projectId,
            'PhoneNumbers.0' => $this->formatPhone($to),
            'SigContent' => $data['sign_name'] ?? $this->signName,
            'TemplateId' => $data['template_id'] ?? '',
        ];

        $templateParams = $data['template_params'] ?? [];
        foreach ($templateParams as $index => $value) {
            $params['Params.' . $index] = $value;
        }

        $params['Signature'] = $this->computeSignature($params);

        try {
            $response = $this->request('GET', self::API_URL . '?' . http_build_query($params));
            return $this->parseResponse($response);
        } catch (GatewayException $e) {
            return $this->buildErrorResponse($e->getMessage(), $e->getCode(), $e->getContext());
        }
    }

    /**
     * 发送模板短信
     * 
     * 使用预定义模板发送短信
     * 
     * @param string $to 接收手机号
     * @param string $templateId 模板 ID
     * @param array $params 模板参数
     * @return array 发送结果
     */
    public function sendTemplate(string $to, string $templateId, array $params = []): array
    {
        return $this->send($to, '', [
            'template_id' => $templateId,
            'template_params' => $params,
        ]);
    }

    /**
     * 批量发送短信
     * 
     * 向多个手机号发送相同内容的短信
     * 
     * @param array $recipients 接收手机号数组
     * @param string $content 短信内容
     * @param array $data 附加数据
     * @return array 发送结果
     */
    public function sendBatch(array $recipients, string $content, array $data = []): array
    {
        $results = [];
        foreach ($recipients as $recipient) {
            $results[] = $this->send($recipient, $content, $data);
        }
        return $this->buildSuccessResponse(['results' => $results]);
    }

    /**
     * 计算签名
     * 
     * 生成 UCloud API 所需的签名
     * 
     * @param array $params 请求参数
     * @return string 签名字符串
     */
    protected function computeSignature(array $params): string
    {
        ksort($params);
        $queryString = http_build_query($params);
        return base64_encode(hash_hmac('sha1', $queryString, $this->privateKey, true));
    }

    /**
     * 解析响应结果
     * 
     * 将 UCloud API 响应转换为标准化的结果格式
     * 
     * @param array $response API 响应
     * @return array 标准化的响应结果
     */
    protected function parseResponse(array $response): array
    {
        if (isset($response['RetCode']) && $response['RetCode'] === 0) {
            return $this->buildSuccessResponse([
                'session_no' => $response['SessionNo'] ?? '',
            ]);
        }

        return $this->buildErrorResponse(
            $response['Message'] ?? '发送失败',
            $response['RetCode'] ?? 500,
            $response
        );
    }
}
